#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================
#-----------------------------------------------------------------------------------------------
# Script Function and Description:
#
# Master script to set-write-suspend (freeze) or set-write-resume (thaw) or purging archive logs 
# either for one particular DB2 LUW database (or) multiple databases in an active instance on the server
# or for all databases within all active instances in the server, thus making it possible to initiate
# the master script either as root, needing affect all active instance(s) or as a db2 instance owner, 
# for one particular or many databases within the instance. 
#
#-----------------------------------------------------------------------------------------------
# Assumptions:
#
# This script will need be initiated within a DB2 LUW server either as root user or one of the  
# DB2 LUW database instance user, comprising of one or many databases within.   
#-----------------------------------------------------------------------------------------------
# Overview
# ========
#
# Main process
# ------------
#
# Function descriptions
# =====================
#
#
###############################################################################
## Set up and Initialize variables
###############################################################################
#set -x

INFO=0
WARN=4
ERROR=8

## DB2_NODE=0


INST_OPER='TEST'
SCRIPTNAME=$(basename $0)
LOG_FILE_NAME=db2backup
CALL_SCR_DIR=$(cd `dirname $0` && pwd)
INST_CALL_SCR="act_pre_post.sh"
USRID=$(whoami)
RC=0
MSG_LOG_DIR='/act/scripts/log'
TS=$(date +'%C%y%m%d')
MSGFILE="${MSG_LOG_DIR}/${LOG_FILE_NAME}.$TS.log"
DB2_FOUND=0
SCRLOG_RETN=7

##############################################
#---------------------------------------------------------------------
#Function : handle_signal
#---------------------------------------------------------------------
function handle_signal
{
   echo "Signal received"
   exit 0
}

#---------------------------------------------------------------------
#Function : message_rep
#---------------------------------------------------------------------
function message_rep
{
typeset DATE=$(date +%Y"/"%m"/"%d)
typeset TIME=$(date +%H":"%M":"%S)
typeset MESSAGE_LEVEL=$1
typeset MESSAGE_TEXT=$2

if [[ ${MESSAGE_LEVEL} -eq 0 ]]; then
        echo -e "${DATE} ${TIME}: ${SCRIPTNAME} : INFO : ${MESSAGE_TEXT}"|tee -a $MSGFILE
elif [[ ${MESSAGE_LEVEL} -le 4 ]]; then
        echo -e "${DATE} ${TIME}: ${SCRIPTNAME} : WARN : ${MESSAGE_TEXT}"|tee -a $MSGFILE
elif [[ ${MESSAGE_LEVEL} -le 8 ]]; then
        echo -e "${DATE} ${TIME}: ${SCRIPTNAME} : ERROR: ${MESSAGE_TEXT}"|tee -a $MSGFILE
fi

} # End message_rep


#---------------------------------------------------------------------
#Function : display_usage - Script usage display function
#---------------------------------------------------------------------
function display_usage
{
echo "\
SYNTAX: ${SCRIPTNAME} [-u root (or) db2_instance_owner] [-i db2_instance_owner] [-d db2_database_name] [-o operation] [-h help]


-u:     <root> or <db2-instance-userid> (Default is current user)
-i:     <db2-instance-userid>
-d:     <db2-database-name>
-o:     executing operation, whether <SUSPEND>, <RESUME>, <TEST> or <PRUNE> (Default is TEST)
-h:     Help (display this information)

Operation:

         SUSPEND -> Suspend or Freeze write operations for database(s)
          RESUME -> Resume or thaw write operations for database(s)
           PRUNE -> Prune archive logs of database(s) for a set retention period
            TEST -> Tests this script to display name of database(s) it covers at a given point

Examples:

         ${SCRIPTNAME} -u root -o SUSPEND
         ${SCRIPTNAME} -u root -i db2inst4 -d QADB -o SUSPEND
         ${SCRIPTNAME} -u root -i db2inst4 -o RESUME
         ${SCRIPTNAME} -u db2admin -o RESUME
         ${SCRIPTNAME} -u db2inst1 -o PRUNE
         ${SCRIPTNAME} -u db2inst2 -d PRODDB -o SUSPEND
         ${SCRIPTNAME} -u db2inst6 -d DEVDB -o PRUNE
         ${SCRIPTNAME} -u db2inst1 -o TEST
"
} # End display_usage



#---------------------------------------------------------------------
#Function : call_instance - db2 instances call function to enumerate 
#           instances and spawn script async to enum dbs
#---------------------------------------------------------------------
function call_instance
{
  message_rep ${INFO} "DB2 ${SCRIPTNAME} $INST_OPER - capturing running db2 instances as $USRID\\n"
  if [ "$INST_UID" = "" ]
  then
     message_rep ${INFO} "Running db2sysc instances: $db2_inst\\n"
  else
     db2_inst=$INST_UID
  fi 
  for instdb in $db2_inst; do
    message_rep ${INFO} "Executing Call: ${CALL_SCR_DIR}/${INST_CALL_SCR} $INST_OPER $instdb $INST_DBNAME \\n"
	if [[ "$USRID" = "root" ]] ;  then
		su - $instdb -c "${CALL_SCR_DIR}/${INST_CALL_SCR} $INST_OPER $instdb $INST_DBNAME" >> $MSGFILE 2>&1
	else
		/usr/bin/sudo -u $instdb -n  ${CALL_SCR_DIR}/${INST_CALL_SCR} $INST_OPER $instdb $INST_DBNAME >> $MSGFILE 2>&1
	fi
	RC=$?
	if [ $RC = 0 ] ; then
		message_rep ${INFO} "${CALL_SCR_DIR}/${INST_CALL_SCR} $INST_OPER $instdb $INST_DBNAME completed successfully.\\n"
	else
		message_rep ${ERROR} "${CALL_SCR_DIR}/${INST_CALL_SCR} $INST_OPER $instdb $INST_DBNAME errored out. Please check output logs.\\n"
		exit ${ERROR}
	fi

  done
  return 0
}

#---------------------------------------------------------------------
#Function : call_scrlog_retn - script log file cleanup 
#---------------------------------------------------------------------
function call_scrlog_retn
{
  message_rep ${INFO} "cleaning up old log files: \\n"
  find ${MSG_LOG_DIR} -name "${LOG_FILE_NAME}*.log" -type f -mtime +$SCRLOG_RETN -print -exec rm -f {} \; >> $MSGFILE

  return 0
}


#############################################################################
## Main process
#############################################################################

trap handle_signal 2 15

##---------------------------------------------------------------------------
## Verify Input parameters to script
##---------------------------------------------------------------------------

if [ $# -eq 0 ]
then
  message_rep ${ERROR} ": No options specified!\\n"
  display_usage
  exit ${ERROR}
fi

if [ ! -f $MSGFILE ]; then
   echo "$(date +"%Y-%m-%d %H:%M:%S") - creating log file..."
   echo " " > $MSGFILE
   chmod a+rwx  $MSGFILE
fi




db2_inst=`ps -ef | grep -e db2sysc | grep -v -e grep | awk '{print $1}' | sort -u`

while [ "$1" != "" ]
  do
   case "$1" in
         ("-u") shift
           USRID=$1
           USRID=`echo $USRID | awk '{print tolower($1)}'`
           if [ "$USRID" = "" ]
           then
             message_rep ${ERROR} "No db2 instance or root userid specified, but -u param supplied!\\n"
             display_usage
           exit ${ERROR}
           fi

           instdb_num=0
           for instdb in root $db2_inst; do
              [[ "$instdb" == "$USRID" ]] && instdb_num=1 && break
           done
           if [ $instdb_num = 0 ]
           then
             message_rep ${INFO} "Running db2sysc instances: $db2_inst\\n"
             message_rep ${ERROR} "Neither a valid running db2 instance or root userid specified, with -u param!\\n"
             display_usage
           exit ${ERROR}
           fi

           shift;;
         ("-s") shift
           SUSRID=$1
           SUSRID=`echo $SUSRID | awk '{print tolower($1)}'`
           if [ "$SUSRID" = "" ]
           then
             message_rep ${ERROR} "No db2 instance or root userid or super user is specified, but -s param supplied!\\n"
             display_usage
           exit ${ERROR}
           fi

           instdb_num=0
           for instdb in $SUSRID $db2_inst; do
              [[ "$instdb" == "$SUSRID" ]] && instdb_num=1 && break
           done
           if [ $instdb_num = 0 ]
           then
             message_rep ${INFO} "Running db2sysc instances: $db2_inst\\n"
             message_rep ${ERROR} "Neither a valid running db2 instance or super userid specified, with -s param!\\n"
             display_usage
           exit ${ERROR}
           fi

           shift;;
         ("-i") shift
           INST_UID=$1
           INST_UID=`echo $INST_UID | awk '{print tolower($1)}'`
           if [ "$INST_UID" = "" ]
           then
             message_rep ${ERROR} "No db2 instance userid specified, but -i param supplied!\\n"
             display_usage
           exit ${ERROR}
           fi

           instdb_num=0
           for instdb in $db2_inst; do
              [[ "$instdb" == "$INST_UID" ]] && instdb_num=1 && break
           done
           if [ $instdb_num = 0 ]
           then
             message_rep ${INFO} "Running db2sysc instances: $db2_inst\\n"
             message_rep ${ERROR} "An invalid running db2 instance userid specified, with -i param!\\n"
             display_usage
           exit ${ERROR}
           fi

           shift;;
         ("-d") shift
           INST_DBNAME=$1
           if [ "$INST_DBNAME" = "" ]
           then
             message_rep ${ERROR} "No db2 database name specified, but -d param supplied!\\n"
             display_usage
           exit ${ERROR}
           fi
           shift;;
         ("-o") shift
           INST_OPER=$1
           if [ "$INST_OPER" = "" ]
           then
             message_rep ${ERROR} "No operation to execute specified, but -o param supplied!\\n"
             display_usage
           exit ${ERROR}
           fi
      if [[ ("$INST_OPER" != "SUSPEND") && ("$INST_OPER" != "RESUME") && ("$INST_OPER" != "PRUNE") && ("$INST_OPER" != "TEST") ]]
           then
             message_rep ${ERROR} "An invalid operation to execute specified!\\n"
             display_usage
           exit ${ERROR}
           fi
           shift;;
    ("-h") shift; display_usage; exit ${ERROR};;
    (*)    shift; message_rep ${ERROR} "Invalid Parameter Specified\\n"; display_usage; exit ${ERROR};;
   esac
  done


##---------------------------------------------------------------------------
## Based on root or db2 instance id initiate execution call for relevant db operation
##---------------------------------------------------------------------------


#if [[ "$USRID" = "root" ]]
if [[ "$USRID" = "$SUSRID" ]] || [[ "$USRID" = "root" ]]
then

   call_instance

else

   ${CALL_SCR_DIR}/${INST_CALL_SCR} $INST_OPER $USRID $INST_DBNAME >> $MSGFILE 2>&1
   RC=$?
   if [ $RC = 0 ] ; then
      message_rep ${INFO} "${CALL_SCR_DIR}/${INST_CALL_SCR} $INST_OPER $USRID $INST_DBNAME completed successfully.\\n"
   else
      message_rep ${ERROR} "${CALL_SCR_DIR}/${INST_CALL_SCR} $INST_OPER $USRID $INST_DBNAME errored out. Please check output logs.\\n"
      exit ${ERROR}
   fi

fi

if [ $RC = 0 ] ; then
   call_scrlog_retn
fi

exit 0
