#!/bin/bash

#set -eo pipefail

source /act/scripts/mongodbbackup.conf

dbhost=$hostip
dbport=$mport
backupdir=$logbackupdir

if [ -z $dbhost ]; then
   echo "ERROR dbhost IP is not set , check /act/scripts/mongodbbackup.conf file existing..."
   exit 1
fi

if [ -z $dbport ]; then
   echo "ERROR mongdb port is not set, check /act/scripts/mongodbbackup.conf file existing..."
   exit 1
fi

if [ ! -d $backupdir ]; then
   echo "ERROR mongdb logbackupdir is not set, check /act/scripts/mongodbbackup.conf file  exiting ...."
   exit 1
fi

# Host name (or IP address) of mongo server e.g localhost
DBHOST="$dbhost"

# Port that mongo is listening on
DBPORT="$dbport"

# Backup directory location e.g /backups
BACKUPDIR="$backupdir"

#
# MongoDB Backup Script
#=====================================================================
#=====================================================================
# Set the following variables to your system needs
# (Detailed instructions below variables)
#=====================================================================

# Do you want to do hourly backups? How long do you want to keep them?
DOHOURLY="yes"
HOURLYRETENTION=24

# ============================================================
# === ADVANCED OPTIONS ( Read the doc's below for details )===
#=============================================================

# Choose Compression type. (gzip or bzip2)
COMP="gzip"

# Choose if the uncompressed folder should be deleted after compression has completed
CLEANUP="yes"

# Additionally keep a copy of the most recent backup in a seperate directory.
LATEST="no"

# Make Hardlink not a copy
LATESTLINK="no"

# Use oplog for point-in-time snapshotting.
OPLOG="yes"

# Choose other Server if is Replica-Set Master
REPLICAONSLAVE="yes"

# Allow DBUSERNAME without DBAUTHDB
REQUIREDBAUTHDB="no"

shellout () {
    if [ -n "$1" ]; then
        echo "$1"
        exit 1
    fi
    exit 0
}

# External config - override default values set above
for x in default sysconfig; do
  if [ -f "/etc/$x/automongobackup" ]; then
      # shellcheck source=/dev/null
      source /etc/$x/automongobackup
  fi
done

#=====================================================================

PATH=/usr/local/bin:/usr/bin:/bin
DATE=$(date +%Y-%m-%d_%Hh%Mm)                     # Datestamp e.g 2002-09-21
HOD=$(date +%s)                                   # Current timestamp for PITR backup
VER=0.11                                          # Version Number
LOGFILE=$BACKUPDIR/$DBHOST-$(date +%H%M).log       # Logfile Name
LOGERR=$BACKUPDIR/ERRORS_$DBHOST-$(date +%H%M).log # Logfile Name
OPT=""                                            # OPT string for use with mongodump
OPTSEC=""                                         # OPT string for use with mongodump in select_secondary_member function
QUERY=""                                          # QUERY string for use with mongodump
HOURLYQUERY=""                                    # HOURLYQUERY string for use with mongodump

# Do we need to backup only a specific database?
if [ "$DBNAME" ]; then
  OPT="$OPT -d $DBNAME"
fi

if [ "x$DBNAME" = "x" ] && [ "$DOHOURLY" == "yes" ]; then
  OPT="$OPT -d local"
fi

# Do we need to backup only a specific collections?
if [ "$COLLECTIONS" ]; then
  for x in $COLLECTIONS; do
    OPT="$OPT --collection $x"
  done
fi

if [ "x$COLLECTIONS" = "x" ] && [ "$DOHOURLY" == "yes" ]; then
    OPT="$OPT --collection oplog.rs"
fi

# Do we need to exclude collections?
if [ "$EXCLUDE_COLLECTIONS" ]; then
  for x in $EXCLUDE_COLLECTIONS; do
    OPT="$OPT --excludeCollection $x"
  done
fi

# Do we use oplog for point-in-time snapshotting?
if [ "$OPLOG" = "yes" ] && [ -z "$DBNAME" ] && [ "$DOHOURLY" != "yes" ]; then
    OPT="$OPT --oplog"
fi

#mkdir -p $BACKUPDIR/{hourly,daily,weekly,monthly} || shellout 'failed to create directories'

# Do we use a filter for hourly point-in-time snapshotting?
if [ "$DOHOURLY" == "yes" ]; then

  # getting PITR START timestamp
  # shellcheck disable=SC2012
  [ "$COMP" = "gzip" ] && HOURLYQUERY=$(ls -t $BACKUPDIR | head -n 1 | cut -d '.' -f2)

  # setting the start timestamp to NOW for the first execution
  if [ -z "$HOURLYQUERY" ]; then
      QUERY=""
    else
      # limit the documents included in the output of mongodump
      # shellcheck disable=SC2016
     # QUERY='{ "ts" : { $gt :  Timestamp('$HOURLYQUERY', 1) } }'
      QUERY='"{ts : { \"\$gt\" : { \"\$timestamp\" : { \"t\" : $HOURLYQUERY, \"i\" : 1 } } }}"'
  fi
fi

# Create required directories
#mkdir -p $BACKUPDIR/{hourly,daily,weekly,monthly} || shellout 'failed to create directories'

# Do we use a filter for hourly point-in-time snapshotting?
if [ "$DOHOURLY" == "yes" ]; then

  # getting PITR START timestamp
  # shellcheck disable=SC2012
  [ "$COMP" = "gzip" ] && HOURLYQUERY=$(ls -t $BACKUPDIR | head -n 1 | cut -d '.' -f2)

  # setting the start timestamp to NOW for the first execution
  if [ -z "$HOURLYQUERY" ]; then
      QUERY=""
    else
      # limit the documents included in the output of mongodump
      # shellcheck disable=SC2016
      #QUERY='{ "ts" : { $gt :  Timestamp('$HOURLYQUERY', 1) } }'
      QUERY="{ts : { \"\$gt\" : { \"\$timestamp\" : { \"t\" : "$HOURLYQUERY", \"i\" : 1 } } }}"
  fi
fi

# Check for correct sed usage
if [ "$(uname -s)" = 'Darwin' ] || [ "$(uname -s)" = 'FreeBSD' ]; then
    SED="sed -i ''"
else
    SED="sed -i"
fi

# IO redirection for logging.
touch "$LOGFILE"
exec 6>&1           # Link file descriptor #6 with stdout.
                    # Saves stdout.
exec > "$LOGFILE"     # stdout replaced with file $LOGFILE.

touch "$LOGERR"
exec 7>&2           # Link file descriptor #7 with stderr.
                    # Saves stderr.
exec 2> "$LOGERR"     # stderr replaced with file $LOGERR.

# When a desire is to receive log via e-mail then we close stdout and stderr.
[ "x$MAILCONTENT" == "xlog" ] && exec 6>&- 7>&-

# Functions

# Database dump function
dbdump () {
    if [ -n "$QUERY" ]; then
        # filter for point-in-time snapshotting and if DOHOURLY=yes
        # shellcheck disable=SC2086
        echo mongodump --quiet --host=$DBHOST:$DBPORT --out="$1" $OPT -q "$QUERY"
        mongodump  --host=$DBHOST:$DBPORT --out="$1" $OPT -q "$QUERY"
        MDUMPSTATUS=$?
      else
        # all others backups type
        # shellcheck disable=SC2086
        echo mongodump --quiet --host=$DBHOST:$DBPORT --out="$1" $OPT
        mongodump --host=$DBHOST:$DBPORT --out="$1" $OPT
        MDUMPSTATUS=$?
    fi
    if [ $MDUMPSTATUS -ne 0 ]; then
        echo "ERROR: mongodump failed: $1" >&2
        return 1
    fi
    [ -e "$1" ] && return 0
    echo "ERROR: mongodump failed to create dumpfile: $1" >&2
    return 1
}

#
# Select first available Secondary member in the Replica Sets and show its
# host name and port.
#
function select_secondary_member {
    # We will use indirect-reference hack to return variable from this function.
    local __return=$1

    # Return list of with all replica set members
    # shellcheck disable=SC2086
    members=( $(mongo --quiet --host $DBHOST:$DBPORT --eval 'rs.conf().members.forEach(function(x){ print(x.host) })' $OPTSEC ) )

    # Check each replset member to see if it's a secondary and return it.
    if [ ${#members[@]} -gt 1 ]; then
        for member in "${members[@]}"; do

            is_secondary=$(mongo --quiet --host "$member" --eval 'rs.isMaster().secondary' $OPTSEC )
            case "$is_secondary" in
                'true')     # First secondary wins ...
                    secondary=$member
                    break
                ;;
                'false')    # Skip particular member if it is a Primary.
                    continue
                ;;
                *)          # Skip irrelevant entries.  Should not be any anyway ...
                    continue
                ;;
            esac
        done
    fi

    if [ -n "$secondary" ]; then
        # Ugly hack to return value from a Bash function ...
        # shellcheck disable=SC2086
        eval $__return="'$secondary'"
    fi
}

if [ -n "$MAXFILESIZE" ]; then
    write_file() {
        split --bytes "$MAXFILESIZE" --numeric-suffixes - "${1}-"
    }
else
    write_file() {
        cat > "$1"
    }
fi

# Compression function plus latest copy
compression () {
    SUFFIX=""
    dir=$(dirname "$1")
    file=$(basename "$1")
    if [ -n "$COMP" ]; then
        [ "$COMP" = "gzip" ] && SUFFIX=".tgz"
        [ "$COMP" = "bzip2" ] && SUFFIX=".tar.bz2"
        echo Tar and $COMP to "$file$SUFFIX"
        cd "$dir" || return 1
        tar -cf - "$file" | $COMP --stdout | write_file "${file}${SUFFIX}"
        cd - >/dev/null || return 1
    else
        echo "No compression option set, check advanced settings"
    fi

    if [ "$LATEST" = "yes" ]; then
        if [ "$LATESTLINK" = "yes" ];then
            COPY="ln"
        else
            COPY="cp"
        fi
        $COPY "$1$SUFFIX" "$BACKUPDIR/latest/"
    fi

    if [ "$CLEANUP" = "yes" ]; then
        echo Cleaning up folder at "$1"
        rm -rf "$1"
    fi

    return 0
}

# Run command before we begin
if [ "$PREBACKUP" ]; then
    echo ======================================================================
    echo "Prebackup command output."
    echo
    eval "$PREBACKUP"
    echo
    echo ======================================================================
    echo
fi

# Hostname for LOG information
if [ "$DBHOST" = "localhost" ] || [ "$DBHOST" = "127.0.0.1" ]; then
    HOST=$(hostname)
    if [ "$SOCKET" ]; then
        OPT="$OPT --socket=$SOCKET"
    fi
else
    HOST=$DBHOST
fi

# Try to select an available secondary for the backup or fallback to DBHOST.
if [ "x${REPLICAONSLAVE}" == "xyes" ]; then
    # Return value via indirect-reference hack ...
    select_secondary_member secondary

    if [ -n "$secondary" ]; then
        DBHOST=${secondary%%:*}
        DBPORT=${secondary##*:}
    else
        SECONDARY_WARNING="WARNING: No suitable Secondary found in the Replica Sets.  Falling back to ${DBHOST}."
    fi
fi

echo ======================================================================
echo AutoMongoBackup VER $VER

if [ ! -z "$SECONDARY_WARNING" ]; then
    echo
    echo "$SECONDARY_WARNING"
fi

# Hourly Backup
if [[ $DOHOURLY = "yes" ]] ; then
    echo Hourly Backup of Databases
    echo
    # Delete old hourly backups while respecting the set rentention policy.
    if [[ $HOURLYRETENTION -ge 0 ]] ; then
        NUM_OLD_FILES=$(find $BACKUPDIR -depth -not -newermt "$HOURLYRETENTION hour ago" -type f | wc -l)
        if [[ $NUM_OLD_FILES -gt 0 ]] ; then
            echo "Deleting $NUM_OLD_FILES global setting backup file\(s\) made in previous weeks."
            find $BACKUPDIR -not -newermt "$HOURLYRETENTION hour ago" -type f -delete
        fi
    fi
    FILE="$BACKUPDIR/$DATE.$HOD"
    # convert timestamp to date: echo $TIMESTAMP | gawk '{print strftime("%c", $0)}'

fi

# FILE will not be set if no frequency is selected.
if [[ -z "$FILE" ]] ; then
  echo "ERROR: No backup frequency was chosen."
  echo "Please set one of DOHOURLY,DODAILY,DOWEEKLY,DOMONTHLY to \"yes\"" 
  exit 1
fi

dbdump "$FILE" && compression "$FILE"

echo ----------------------------------------------------------------------
echo "Backup End Time $(date)"
echo ======================================================================

echo Total disk space used for backup storage..
echo Size - Location
du -hs "$BACKUPDIR"
echo
echo ======================================================================

# Run command when we're done
if [ "$POSTBACKUP" ]; then
    echo ======================================================================
    echo "Postbackup command output."
    echo
    eval "$POSTBACKUP"
    echo
    echo ======================================================================
fi

# TODO: Would be nice to know if there were any *actual* errors in the $LOGERR
STATUS=0
if [ -s "$LOGERR" ]; then
    STATUS=1
fi

# Clean up Logfile
rm -f "$LOGFILE" "$LOGERR"

exit $STATUS

